// Copyright (C) 2014-2017 Bayerische Motoren Werke Aktiengesellschaft (BMW AG)
// This Source Code Form is subject to the terms of the Mozilla Public
// License, v. 2.0. If a copy of the MPL was not distributed with this
// file, You can obtain one at http://mozilla.org/MPL/2.0/.

#ifndef VSOMEIP_LOGGER_HPP
#define VSOMEIP_LOGGER_HPP

#include <string>
#include <sstream>

#ifdef _WIN32
#include <iostream>
#endif

#include <vsomeip/export.hpp>

#include <boost/log/sources/severity_logger.hpp>
#include <boost/log/trivial.hpp>
#include "HsLogn.h"

#define LOG_TAG "vsomeip"

namespace vsomeip {

class VSOMEIP_IMPORT_EXPORT logger {
public:
    static std::shared_ptr<logger> get();

    virtual ~logger() {
    }

    virtual boost::log::sources::severity_logger_mt<
            boost::log::trivial::severity_level> & get_internal() = 0;
};

typedef enum {
    HSLOG_LEVEL_WTF = 0x0,
    HSLOG_LEVEL_E = 0x1,
    HSLOG_LEVEL_W = 0x2,
    HSLOG_LEVEL_I = 0x3,
    HSLOG_LEVEL_D = 0x4,
    HSLOG_LEVEL_V = 0x5,
} hslog_level_entry_e;

class HsLogger
{
    std::string _file;
    int _line;
    std::string _function;
    std::string _tag;
    std::stringstream _message;
    hslog_level_entry_e _entry;
public:
    HsLogger(const char* const file, const int line, const char* const function, const char* const tag, hslog_level_entry_e entry)
    {
        _file = file;
        _line = line;
        _function = function;
        _tag = tag;
        _entry = entry;
    }

    ~HsLogger()
    {
        switch (_entry) {
            case HSLOG_LEVEL_WTF:
                HsLognWTF(_file.c_str() , _line, _function.c_str(), _tag.c_str(), _message.str().c_str());
                break;
            case HSLOG_LEVEL_E:
                HsLognE(_file.c_str() , _line, _function.c_str(), _tag.c_str(), _message.str().c_str());
                break;
            case HSLOG_LEVEL_W:
                HsLognW(_file.c_str() , _line, _function.c_str(), _tag.c_str(), _message.str().c_str());
                break;
            case HSLOG_LEVEL_I:
                HsLognI(_file.c_str() , _line, _function.c_str(), _tag.c_str(), _message.str().c_str());
                break;
            case HSLOG_LEVEL_D:
                HsLognD(_file.c_str() , _line, _function.c_str(), _tag.c_str(), _message.str().c_str());
                break;
            case HSLOG_LEVEL_V:
                HsLognV(_file.c_str() , _line, _function.c_str(), _tag.c_str(), _message.str().c_str());
                break;
            default:
                break;
        }
    }
    
    template<typename T>
    HsLogger& operator<<(const T& value)
    {
        std::stringstream tmp;
        try {
            tmp << value;
            if ((_message.str().capacity() + tmp.str().capacity()) < _message.str().max_size()) {
                _message << value;
            }
        } catch (const std::exception& e) {
            HsLognE(_file.c_str() , _line, _function.c_str(), _tag.c_str(), e.what());
        }
        return *this;
    }
};

#define VSOMEIP_FATAL BOOST_LOG_SEV(vsomeip::logger::get()->get_internal(), \
                boost::log::trivial::severity_level::fatal)
#define VSOMEIP_ERROR BOOST_LOG_SEV(vsomeip::logger::get()->get_internal(), \
                boost::log::trivial::severity_level::error)
#define VSOMEIP_WARNING BOOST_LOG_SEV(vsomeip::logger::get()->get_internal(), \
                boost::log::trivial::severity_level::warning)
#define VSOMEIP_INFO BOOST_LOG_SEV(vsomeip::logger::get()->get_internal(), \
                boost::log::trivial::severity_level::info)
#define VSOMEIP_DEBUG BOOST_LOG_SEV(vsomeip::logger::get()->get_internal(), \
                boost::log::trivial::severity_level::debug)
#define VSOMEIP_TRACE BOOST_LOG_SEV(vsomeip::logger::get()->get_internal(), \
                boost::log::trivial::severity_level::trace)
#define HSLOG_ASSERT(tag) HsLogger(__FILE__, __LINE__, __PRETTY_FUNCTION__, tag, HSLOG_LEVEL_WTF)
#define HSLOG_ERROR(tag) HsLogger(__FILE__, __LINE__, __PRETTY_FUNCTION__, tag, HSLOG_LEVEL_E)
#define HSLOG_WARNING(tag) HsLogger(__FILE__, __LINE__, __PRETTY_FUNCTION__, tag, HSLOG_LEVEL_W)
#define HSLOG_INFO(tag) HsLogger(__FILE__, __LINE__, __PRETTY_FUNCTION__, tag, HSLOG_LEVEL_I)
#define HSLOG_DEBUG(tag) HsLogger(__FILE__, __LINE__, __PRETTY_FUNCTION__, tag, HSLOG_LEVEL_D)
#define HSLOG_VERBOSE(tag) HsLogger(__FILE__, __LINE__, __PRETTY_FUNCTION__, tag, HSLOG_LEVEL_V)

} // namespace vsomeip

#endif // VSOMEIP_LOGGER_HPP
